@@ -14,10 +14,10 @@ import readline
 import textwrap
 import subprocess
 
-if sys.version_info.major >= 3:
+if sys.version_info[0] >= 3:
     getinput = input
     # perf_counter = time.perf_counter
-elif sys.version_info.major < 3:
+elif sys.version_info[0] < 3:
     getinput = raw_input
     # perf_counter = time.time
 
@@ -43,6 +43,10 @@ class Screen:
             "file": {
                 "bg": TerminalOperations.background_color(235),
                 "fg": TerminalOperations.foreground_color(39)
+            },
+            "search": {
+                "bg": TerminalOperations.background_color(235),
+                "fg": Todo.colors["search_match"],
             }
         },
         "selected": {
@@ -62,6 +66,7 @@ class Screen:
         self.todo           = todo
         self.sorting_names  = ["Unsorted", "Ascending ", "Descending"]
         self.sorting        = 0
+        self.clear_search_term()
         self.readline_editing_mode = readline_editing_mode
         self.update_todos(todo)
         self.original_terminal_settings = termios.tcgetattr(sys.stdin.fileno())
@@ -108,7 +113,9 @@ class Screen:
             self.move_selection_top()
 
         # load items
-        if self.selected_context == 0 and self.selected_project == 0:
+        if self.search_string:
+            self.items = self.todo.search(self.search_string)
+        elif self.selected_context == 0 and self.selected_project == 0:
             self.items = self.todo.todo_items
         elif self.selected_project != 0 and self.selected_context != 0:
             self.items = self.todo.filter_context_and_project(self.context_list[self.selected_context], self.project_list[self.selected_project])
@@ -118,13 +125,18 @@ class Screen:
             self.items = self.todo.filter_context(self.context_list[self.selected_context])
 
         # Header
-        left_header_todo_count = " {} Todos ".format( len(self.items) )
-        left_header_sorting    = " Sorting: {} ".format( self.sorting_names[self.sorting] )
+        left_header_todo_count = " {0} Todos ".format( len(self.items) )
+        left_header_sorting    = " Sorting: {0} ".format( self.sorting_names[self.sorting] )
         left_header_size       = len(left_header_todo_count + left_header_sorting)
 
-        right_header = " {} ".format(
-            self.todo.file_path[:].replace(os.environ['HOME'], '~')
-        ).rjust(columns-left_header_size)[:columns-left_header_size]
+        if self.search_string:
+            right_header = " searching: {0} ".format(
+                self.search_string
+            ).ljust(columns-left_header_size)[:columns-left_header_size]
+        else:
+            right_header = " {0} ".format(
+                self.todo.file_path[:].replace(os.environ['HOME'], '~')
+            ).rjust(columns-left_header_size)[:columns-left_header_size]
 
         term.output( term.clear_formatting() )
         term.move_cursor(1, 1)
@@ -133,6 +145,10 @@ class Screen:
         term.output( Screen.colors["header"]["sorting"]["fg"] + Screen.colors["header"]["sorting"]["bg"] )
         term.output( left_header_sorting )
         term.output( Screen.colors["header"]["file"]["fg"] + Screen.colors["header"]["file"]["bg"] )
+        if self.search_string:
+            term.output( Screen.colors["header"]["search"]["fg"] + Screen.colors["header"]["search"]["bg"] )
+        else:
+            term.output( Screen.colors["header"]["file"]["fg"] + Screen.colors["header"]["file"]["bg"] )
         term.output( right_header )
 
         term.move_cursor(2, 1)
@@ -141,9 +157,9 @@ class Screen:
         term.move_cursor(2, 1)
 
         # Contexts
-        term.output("Context: {}".format("".join(
-            ["{} {} {}".format(
-                Todo.colors["context"]+Screen.colors["selected"]["bg"], c, Screen.colors["header"]["fg"]+Screen.colors["header"]["bg"]) if c == self.context_list[self.selected_context] else " {} ".format(c) for c in self.context_list]
+        term.output("Context: {0}".format("".join(
+            ["{0} {1} {2}".format(
+                Todo.colors["context"]+Screen.colors["selected"]["bg"], c, Screen.colors["header"]["fg"]+Screen.colors["header"]["bg"]) if c == self.context_list[self.selected_context] else " {0} ".format(c) for c in self.context_list]
         )))
 
         term.move_cursor(3, 1)
@@ -153,11 +169,13 @@ class Screen:
 
         # Projects
         term.output( Screen.colors["header"]["fg"] + Screen.colors["header"]["bg"] )
-        term.output("Project: {}".format("".join(
-            ["{} {} {}".format(
-                Todo.colors["project"]+Screen.colors["selected"]["bg"], p, Screen.colors["header"]["fg"]+Screen.colors["header"]["bg"]) if p == self.project_list[self.selected_project] else " {} ".format(p) for p in self.project_list]
+        term.output("Project: {0}".format("".join(
+            ["{0} {1} {2}".format(
+                Todo.colors["project"]+Screen.colors["selected"]["bg"], p, Screen.colors["header"]["fg"]+Screen.colors["header"]["bg"]) if p == self.project_list[self.selected_project] else " {0} ".format(p) for p in self.project_list]
         )))
 
+        term.output( term.clear_formatting() )
+
         # Todo List
         if len(self.items) > 0:
             current_item = self.starting_item
@@ -174,11 +192,19 @@ class Screen:
                     else:
                         term.output( term.clear_formatting()+Screen.colors["normal"]["bg"] )
 
-                    term.output(
-                        self.items[current_item].highlight(
-                            self.items[current_item].raw.strip()[:columns].ljust(columns)
+                    # normal output if there is no search string or hit enter after searching
+                    if self.search_string_entered or self.search_string == "":
+                        term.output(
+                            self.items[current_item].highlight(
+                                self.items[current_item].raw.strip()[:columns].ljust(columns)
+                            )
+                        )
+                    else:
+                        term.output(
+                            self.items[current_item].highlight_search_matches(
+                                self.items[current_item].raw.strip()[:columns].ljust(columns)
+                            )
                         )
-                    )
                     term.output( term.clear_formatting() )
                     current_item += 1
         else:
@@ -307,6 +333,16 @@ class Screen:
                 C            - select the previous context
                 s            - switch sorting method
 
+            ### Searching
+
+                /            - start search
+                ctrl-l       - clear search
+
+            ### While Searching
+
+                enter        - end search
+                ctrl-c       - cancel search
+
             ### Manipulating Todo Items
 
                 x            - complete / un-complete selected todo item
@@ -334,6 +370,56 @@ class Screen:
         finally:
             self.set_raw_input()
 
+    def draw_search_prompt(self):
+        self.terminal.output( self.terminal.clear_formatting() )
+        self.terminal.output( Screen.colors["normal"]["fg"]+Screen.colors["normal"]["bg"] )
+        self.terminal.move_cursor(self.terminal.rows, 1)
+        self.terminal.output(" "*self.terminal.columns)
+        self.terminal.move_cursor(self.terminal.rows, 1)
+        self.terminal.output( "/" )
+        self.terminal.output( self.search_string )
+        sys.stdout.flush()
+
+    def clear_search_term(self):
+        self.search_string_entered = False
+        self.search_string = ""
+
+    def unfinalize_search(self):
+        self.search_string_entered = False
+
+    def finalize_search(self):
+        self.search_string_entered = True
+
+    def search_loop(self):
+        self.unfinalize_search()
+        self.move_selection_top()
+        self.selected_context = 0
+        self.selected_project = 0
+        self.update()
+        self.draw_search_prompt()
+        while True:
+            if sys.stdin in select.select([sys.stdin], [], [], 0.1)[0]:
+                c = sys.stdin.read(1)
+                if ord(c) == 3: # ctrl-c
+                    self.clear_search_term()
+                    self.update()
+                    break
+                elif ord(c) == 13: # enter
+                    self.finalize_search()
+                    self.update()
+                    break
+                elif ord(c) == 127: # backspace
+                    if len(self.search_string) > 0:
+                        self.search_string = self.search_string[:-1]
+                    else:
+                        self.clear_search_term()
+                    self.update()
+                    self.draw_search_prompt()
+                else: #TODO only append c if c is a printable character
+                    self.search_string += c
+                    self.update()
+                    self.draw_search_prompt()
+
     def main_loop(self):
         self.set_raw_input()
         self.update()
@@ -390,8 +476,12 @@ class Screen:
                         self.edit_item(new='insert_after')
                     elif c == 'D':
                         self.delete_item()
+                    elif c == '/':
+                        self.search_loop()
                     elif ord(c) == 3: # ctrl-c
                         break
+                    elif ord(c) == 12: # ctrl-l
+                        self.clear_search_term()
                     # elif ord(c) == 127: # backspace
                     # elif ord(c) == 9: # tab
                     elif ord(c) == 27: # special key - read another byte
@@ -468,7 +558,7 @@ class Screen:
         self.terminal.move_cursor(starting_row, 1)
 
         # setup readline
-        readline.parse_and_bind('set editing-mode {}'.format(self.readline_editing_mode))
+        readline.parse_and_bind('set editing-mode {0}'.format(self.readline_editing_mode))
         completer = Completer(self.context_list + self.project_list)
         # we want to autocomplete tokens with @ and + symbols so we remove them from # the current readline delims
         delims = readline.get_completer_delims().replace("@","").replace("+","")
